#############################################################################
# fontreq.tcl Copyright 2004, 2005 r0lZ
# Font requester.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

# example usage:
# set newfont [::fontreq::fontreq .fr {Courier 10} true]

namespace eval ::fontreq {
namespace export fontreq

proc fontreq {w current_font fixed_size_only} {
	if {! $fixed_size_only} {
		set ::fontreq::fontlist [lsort [font families]]
	} else {
		set :fontlist ""
		foreach f [font families] {
			if {[font metrics \"$f\" -fixed]} {
				lappend fontlist $f
			}
		}
		set ::fontreq::fontlist [lsort $fontlist]
	}

	set ::fontreq::family [font actual $current_font -family]
	set ::fontreq::size [font actual $current_font -size]

	set fontindex 0
	for {set i 0} {$i < [llength $::fontreq::fontlist]} {incr i} {
		if {$::fontreq::family == [lindex $::fontreq::fontlist $i]} {
			set fontindex $i
			break
		}
	}

	toplevel $w -class Dialog
	wm withdraw $w
	wm protocol $w WM_DELETE_WINDOW {set ::fontreq::done 0}
	bind $w <Escape> "set ::fontreq::done 0"
	if {$::config(tk) != "aqua"} {wm transient $w .}
	if {$::tcl_platform(platform) == "windows"} {catch {wm attributes $w -toolwindow 1}}
	if {$fixed_size_only} {
		wm title $w "Select fixed-width font"
	} else {
		wm title $w "Select font"
	}

	label $w.oldfont -height 3 -text "Current font: \"[lindex $current_font 0]\" [lindex $current_font 1]\n\nSelect new font..." \
			-anchor w -justify left
	pack  $w.oldfont -side top -fill x -padx 8

	frame $w.fpicker
	pack  $w.fpicker -side top -fill x -expand 1 -padx 8

	listbox $w.fpicker.lb -listvariable ::fontreq::fontlist \
			-height 8 -width 30 -yscrollcommand "$w.fpicker.yscr set" \
			-selectmode single
	scrollbar $w.fpicker.yscr -command "$w.fpicker.lb yview"
	pack  $w.fpicker.lb -side left
	pack  $w.fpicker.yscr -side left -fill y
	bind  $w.fpicker.lb <1> "::fontreq::setFontFamily $w"

	frame $w.fpicker.pad
	pack  $w.fpicker.pad -side left -padx 4

	frame $w.fpicker.f
	pack  $w.fpicker.f -side left -fill y
	label $w.fpicker.f.size -text "Size:"
	pack  $w.fpicker.f.size
	spinbox $w.fpicker.f.spin -from 6 -to 18 -width 2 \
			-textvariable ::fontreq::size -command "::fontreq::setFont $w" \
			-state readonly
	pack  $w.fpicker.f.spin -side top -fill x
	button $w.fpicker.f.cancel -text " Cancel " \
			-command {set ::fontreq::done 0}
	pack  $w.fpicker.f.cancel -side bottom -fill x
	button $w.fpicker.f.ok -text "OK" \
			-command {set ::fontreq::done 1}
	pack  $w.fpicker.f.ok  -side bottom -fill x -pady 4

	label $w.fexample -borderwidth 2 \
			-relief groove -width 32 -height 4 -text [join [list \
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
		"abcdefghijklmnopqrstuvwxyz" \
		"0123456789~`!@#$%^&*()_-+=" \
		"{}[]:;\"'<>,.?/"] "\n"]
	pack  $w.fexample -fill x -expand 1 -padx 8 -pady 8
	if {$::config(platform) == "mac"} {
		pack [frame $w.macpad] -pady 4
	}

	$w.fpicker.lb activate $fontindex
	$w.fpicker.lb selection set $fontindex
	$w.fpicker.lb see $fontindex
	setFont $w

	update
	set wx [expr {[winfo rootx .]+(([winfo width  .]-[winfo reqwidth  $w])/2)}]
	set wy [expr {[winfo rooty .]+(([winfo height .]-[winfo reqheight $w])/2)-25}]
	if {$wx < 10} {set wx 10}
	if {$wy < 22} {set wy 22}
	wm geometry $w +$wx+$wy

	wm deiconify $w
	grab set $w

	set ::fontreq::done -1
	tkwait variable ::fontreq::done
	grab release $w
	destroy $w
	if {$::fontreq::done} {
		return [list $::fontreq::family $::fontreq::size]
	} else {
		return ""
	}
}

proc setFontFamily {w} {
	set x [expr {[winfo pointerx $w]-[winfo rootx $w.fpicker.lb]}]
	set y [expr {[winfo pointery $w]-[winfo rooty $w.fpicker.lb]}]
	set ::fontreq::family "[$w.fpicker.lb get @$x,$y]"
	::fontreq::setFont $w
}

proc setFont {w} {
	$w.fexample configure -font [list $::fontreq::family $::fontreq::size]
}

}

